module CovaIO

		def _io_init
			@dir_user_home 			= File.expand_path("~")
			@dir_cova_home 			= File.expand_path("~/.cova")
			@dir_cova_ios_mods_home = File.expand_path("~/.cocoapods/cova")
			@dir_cocoapods_home 	= File.expand_path("~/.cocoapods")
			@dir_xctool_home 		= File.expand_path("~/.cova/xctool")
			@dir_xcode_build_home 	= File.expand_path("~/Library/Developer/Xcode/DerivedData")
			@dir_cova_apps_home 	= File.expand_path("~/.cova/apps")

			@repo_cova 			    = "git://github.com/dancali/cova.git"
			@repo_cova_ios_mods		= "git://github.com/dancali/cova-ios-mods.git"
			@repo_xctool			= "git://github.com/facebook/xctool.git"
		end

		def _io_exec(cmd)
			pipe = IO.popen(cmd)
			puts cmd
			while (line = pipe.gets)
			  print line		
			end
		end

		def _io_findport(port)
			ok = false
			times = 100
			until ok
				begin
					if times <= 0
						port = -1
						ok = true
					else
				    	socket = TCPSocket.new "localhost", port
						socket.close
						port = port + 1
						times = times - 1
					end
		        rescue Errno::ECONNREFUSED, Errno::ETIMEDOUT
		        	ok = true
		        end
	        end
	        return port
		end

		def _io_exec_xctool(cmd="build", config="Debug", sdk="iphonesimulator")
			_io_exec @dir_xctool_home + "/xctool.sh -workspace cova.xcworkspace -scheme Cova -configuration #{config} -sdk #{sdk} #{cmd}"
		end

		def _io_pod_add_repo(name, repo)
			_io_exec("pod repo add #{name} #{repo}")
			return _io_isdir(@dir_cocoapods_home + "/" + name)
		end

		def _io_isdir(dir,expected=true)
			_log_line (expected ? "Expecting #{dir} to exist" : "Ensuring #{dir} does not exist")
			if File.directory?(dir)
				expected ? _log_line_ok : _log_line_error
		  		return true
		  	end		  	

		  	if expected
			  	_log_line_error
			else
				_log_line_ok
			end
		  	return false
		end

		def _io_isfile(file,expected=true)
			_log_line (expected ? "Expecting #{file} to exist" : "Ensuring #{file} does not exist")
			if File.exist?(file)
				expected ? _log_line_ok : _log_line_error
		  		return true
		  	end		  	

		  	if expected
			  	_log_line_error
			else
				_log_line_ok
			end
		  	return false
		end

		def _io_app_exists(name)
			return File.directory?(@dir_cova_apps_home + "/" + name)
		end

		def _io_open_app_source(name)
			if not _io_app_exists name
				_log_info "Looks like the app #{name} does not exist. Please create if first."
				_log_line_error
				return false
			end

			app_project = "#{@dir_cova_apps_home}/#{name}/ios/cova.xcworkspace"
			if not File.exist?(app_project)
				_log_info "Looks like the app #{name} was never build. Please build it first."
				return false
			end

			_io_exec "open #{app_project}"
			return true
		end

		def _io_ls(dir)
			Dir.foreach(dir) {|entry| puts "#{entry}" unless (entry =='.' || entry == '..') }
		end

		def _io_clone_repo (repo, dir)
			pwd = Dir.pwd
			Dir.chdir(dir)
			_io_exec "git clone #{repo}"
			reponame = File.basename(repo, ".*")
			Dir.chdir(reponame)
			#_io_exec "git remote add upstream #{repo}"
			#Dir.chdir(pwd)
			return _io_isdir(dir + "/" + reponame)
		end

		def _io_rmdir(dir)
			_log_line "Removing #{dir}"
			if File.directory?(dir)
				`rm -rf #{dir}`
			  	if File.directory?(dir)
			  		_log_line_error
			  		return false
				end
			end
			_log_line_ok
			return true
		end

		def _io_lndir(src, targ)
			if _io_isdir src and not _io_isdir targ, false
				`ln -s #{src} #{targ}`
				return _io_isdir targ
			end

			return false
		end

		def _io_cpdir(src, targ)
			if _io_isdir src and not _io_isdir targ, false
				`cp -r #{src} #{targ}`
				return _io_isdir targ
			end
			return false
		end

		def _io_cpfile(src, targ)
			if _io_isfile src and not _io_isfile targ, false
				`cp #{src} #{targ}`
				return _io_isfile targ
			end
			return false
		end

		def _io_mkdir(dir)
			if File.directory?(dir)
				return true
			end
			
			_log_line "Creating #{dir}"
			`mkdir #{dir}`
			File.directory?(dir) ? _log_line_ok : _log_line_error
			return File.directory?(dir)
		end
end