class Cova < Thor

	include CovaConfig
	include CovaLog
	include CovaIO

	def initialize(*args)
	  super
	  _io_init
	end

	desc 'status', 'Checks the status of a Cova installation'
	def status
	   _log_block_start "Checking if Cova is installed"
	   if _io_isdir(@dir_cova_home) and _io_isdir(@dir_cova_ios_mods_home)
	   		_log_block_success
	   		return true
	   end
	   _log_block_error
	   _cova_prompt_install
	   return false
	end

	desc 'uninstall', 'Completely uninstalls Cova'
	def uninstall
	   _log_block_start "Uninstalling Cova"
	   _io_rmdir(@dir_cova_home)
	   _io_rmdir(@dir_cova_ios_mods_home)
	   if _io_isdir(@dir_cova_home, false) or _io_isdir(@dir_cova_ios_mods_home, false)
	   		_log_block_error
	   		return false
	   end
	   _log_block_success
	   return true
	end

	desc 'install', 'Installs Cova from scratch'
	def install
	  _log_block_start "Installing Cova"

	  if _io_isdir(@dir_cova_home, false) or _io_isdir(@dir_cova_ios_mods_home, false)
		  _log_info "It looks like Cova is already installed. Please uninstall it first before installing again."	  	 
		  _log_block_error
		  return false
	  end

	  if _io_mkdir @dir_cova_home and _io_mkdir @dir_cova_apps_home and _io_mkdir @dir_cocoapods_home
	  	  if _io_clone_repo @repo_xctool, @dir_cova_home and _io_clone_repo @repo_cova, @dir_cova_home
	  	  	  if _io_pod_add_repo "cova", @repo_cova_ios_mods
			  	  _log_block_success
			  	  puts
  				  puts "Congrats! Cove is now installed. Enjoy and build the next great app sensation!".green
  				  puts
  			  	  puts "For more info check out http://cova.io"
  			 	  puts
		  	  	  return true
		  	  end
		  end
	  end

	  _log_block_error
	  return false
	end

	desc 'list', 'List all exisiting apps'
	def list
		if not _cova_installed
			_cova_prompt_install
			return false
		end

		_io_ls @dir_cova_apps_home
		return true
    end

	desc 'find NAME', 'Looks for an app'
	def find(name)
		if not _cova_installed
			_cova_prompt_install
			return false
		end

		_log_line "Searching for app #{name}"
		if _io_app_exists name
			_log_line_ok
			return true
		end

		_log_line_error
		return false
    end

	desc 'newapp NAME', 'Creates a new app'
    def newapp (name)

	#	`cp -r Cova #{name}`
	#	app_root = "#{name}"
    #	Dir.chdir app_root

    #	_io_exec "pod repo update cova"
	#	_io_exec "pod install"
	#	_io_exec "open Cova.xcworkspace"

#		`mv app.xcodeproj #{name}.xcodeproj`	
 #   	project  = Xcodeproj::Project.new "#{name}.xcodeproj"
#		target = project.targets[0]
 #   	target.name = name
#		project.targets[1].name = "#{name}Tests"

#		debug_build_settings = target.build_configuration_list.build_configurations[1].build_settings

    	# `cp ../cova/cova/cova-Info.plist #{name}-Info.plist`
    	# `cp ../cova/cova/AppDelegate.h AppDelegate.h`
    	# `cp ../cova/cova/AppDelegate.m AppDelegate.m`
    	# `cp ../cova/cova/cova-Prefix.pch #{name}-Prefix.pch`
    	# `cp ../cova/cova/main.m main.m`
		#info_plist_file = project.new_file("app-Info.plist")
		#pch_file = project.new_file("#{name}-Prefix.pch")
		# main_file = project.new_file("main.m")
		# delegate_h_file = project.new_file("AppDelegate.h")
		# delegate_m_file = project.new_file("AppDelegate.m")
		# target.add_file_references([main_file, delegate_h_file, delegate_m_file])
  #       release_build_settings = target.build_configuration_list.build_configurations[0].build_settings
  #       release_build_settings['INFOPLIST_FILE'] = "#{name}-Info.plist"
  #       release_build_settings['GCC_PREFIX_HEADER'] = "#{name}-Prefix.pch"
		# debug_build_settings = target.build_configuration_list.build_configurations[1].build_settings
  #       debug_build_settings['INFOPLIST_FILE'] = "#{name}-Info.plist"
  #       debug_build_settings['GCC_PREFIX_HEADER'] = "#{name}-Prefix.pch"
		# project.add_system_framework "CoreData", target
  #   	project.add_system_framework "UIKit", target
  #   	project.add_system_framework "Foundation", target

#		project.save_as("#{name}.xcodeproj")
#		_io_exec "open #{name}.xcodeproj"

        #debug_build_settings['PRODUCT_NAME'] = name
        #puts 
        #puts debug_build_settings['PRODUCT_NAME'] 

    	#puts project.pretty_print

    	#info_plist = project.main_group.new_file("#{name}-Info.plist")
		#target.add_file_references([info_plist])

    	#debug = project.build_configurations
    	#pp debug

    	#config   = project.configuration_list :ios
		
		#project.add_system_framework "CoreData", target
    	#project.add_system_framework "UIKit", target
    	#project.add_system_framework "Foundation", target

    	#src = project.new_group "src"
    	#res = project.new_group "res"
    	#test = project.new_group "test"

		#File.open("src/main.m", 'w') {|f| f.write("#import <UIKit/UIKit.h>\r\n#import \"AppDelegate.h\"\r\nint main(int argc, char *argv[]) {\r\n@autoreleasepool { return UIApplicationMain(argc, argv, nil, NSStringFromClass([AppDelegate class])); }\r\n}")}
		#File.open("src/AppDelegate.h", 'w') {|f| f.write("#import <UIKit/UIKit.h>\r\n@interface AppDelegate : UIResponder <UIApplicationDelegate>\r\n @property (strong, nonatomic) UIWindow *window; \r\n@property (readonly, strong, nonatomic) NSManagedObjectContext *managedObjectContext; \r\n@property (readonly, strong, nonatomic) NSManagedObjectModel *managedObjectModel; \r\n@property (readonly, strong, nonatomic) NSPersistentStoreCoordinator *persistentStoreCoordinator; \r\n- (void)saveContext; \r\n- (NSURL *)applicationDocumentsDirectory; \r\n@end")}
		#main_file = src.new_file("src/main.m")
		#delegate_file = src.new_file("src/AppDelegate.h")
		
		#project.save_as("#{name}.xcodeproj")

		#File.open("Podfile", 'w') {|f| f.write("platform :ios, '5.0'\r\npod 'covamod-core'")}
		#_io_exec "pod setup"
		#_io_exec "pod repo update cova"
		#_io_exec "pod install"
		#_io_exec "open #{name}.xcworkspace"
		
		#_io_exec "open #{name}.xcodeproj"
    end

	desc 'setup', 'Setup'
    def setup ()
    	_io_exec "curl -fsSL https://raw.github.com/mxcl/homebrew/go"
		_io_exec "brew update"
    end

	desc 'image', 'Creates a new image'
    def image ()
    	png = ChunkyPNG::Image.new(320, 480, ChunkyPNG::Color::TRANSPARENT)
		png[1,1] = ChunkyPNG::Color.rgba(10, 20, 30, 128)
		png[2,1] = ChunkyPNG::Color('black @ 0.5')
		png.save('Default.png', :interlace => true)
    end

	desc 'create [-b] NAME', 'Creates a new Cova app (-b to build it including tests)'
	option :build, :type => :boolean, :aliases => :b
    def create (name)

		if not _cova_installed
			_cova_prompt_install
			return false
		end

      	appname = name.strip.delete(' ')
      	_log_block_start "Creating Cova app #{appname}"
      	dir = Dir.pwd + "/" + appname

      	if _io_isdir dir, false or _io_app_exists appname or not _io_mkdir dir or not _io_lndir dir, @dir_cova_apps_home + "/" + appname
      		_log_block_error
      		return false
      	end

      	if not _io_cpdir @dir_cova_home + "/cova/store", dir + "/store"
			_log_block_error
      		return false
      	end

      	if _io_cpdir @dir_cova_home + "/cova/web/app", dir + "/www"
      		config_content = File.read(dir + "/www/config.php")
      		config_content = config_content.gsub(/%cova_home%/, @dir_cova_home)
			File.open(dir + "/www/config.php", 'w') { |file| file.puts config_content }
      	end

      	if _io_cpdir @dir_cova_home + "/cova/ios", dir + "/ios"
	    	Dir.chdir(@dir_cova_apps_home + "/" + name + "/ios")
  		   	
  		   	#project  = Xcodeproj::Project.new "Cova.xcodeproj"
			#target = project.targets[0]
			#test_target = project.targets[1]
			#test_debug_build_settings = test_target.build_configuration_list.build_configurations[1].build_settings

			# puts test_debug_build_settings
   			# target.name = name
			# project.targets[1].name = "#{name}Tests"
			
			#release_build_settings = target.build_configuration_list.build_configurations[0].build_settings
			#debug_build_settings = target.build_configuration_list.build_configurations[1].build_settings
			#project.save_as("Cova.xcodeproj")

      		if options[:build]
		     	if not _cova_build_dependencies
		     		_log_line "Attempted to build the app dependencies"
		     		_log_line_error
	    	 		return false
	     		end
      			
				_io_exec_xctool "test"
				_io_open_app_source name
      			return true
      		end

    	  	_log_block_success
	      	return true
      	end

		_log_block_error
      	return false
    end

    desc 'bye NAME', 'Say bye to an app and stop responding to activities'
    def bye (name)
    	if not _cova_installed
			_cova_prompt_install
			return false
		end

    	if not _io_app_exists name
    		_log_line "App #{name} does not exist."
    		_log_line_error
    		return false
    	end

		pid_file = @dir_cova_apps_home + "/" + name + "/store/hi.lock"
		if not _io_isfile pid_file
			_log_info "How rude. Say hi first."
			return false
		end

		File.open(pid_file).each_line {|pid| Process.kill("TERM", pid.to_i)}
		`rm -rf #{pid_file}`

		if not _io_isfile pid_file, false
			_log_info "Bye!"
			return true
		end

		_log_line_error
		return false    	
    end

    desc 'log NAME', 'A log of all activities'
    def log (name)
    	if not _cova_installed
			_cova_prompt_install
			return false
		end

    	if not _io_app_exists name
    		_log_line "App #{name} does not exist"
    		_log_line_error
    		return false
    	end

		db_file = @dir_cova_apps_home + "/" + name + "/store/log.db"
		if File.exist?(db_file)
 			db = SQLite3::Database.new db_file
			db.execute "select * from changelog" do |row|
				puts "[" + row[2] + "] " + row[1] + " " + row[0]
			end
			return true
		end

		_log_info "Looks like there's no activity yet. So do something first."
		return false
    end

    desc 'hi NAME', 'Say hi to an app and respond to activities'
    def hi (name)
    	if not _cova_installed
			_cova_prompt_install
			return false
		end

    	if not _io_app_exists name
    		_log_line "App #{name} does not exist"
    		_log_line_error
    		return false
    	end

		pid_file = @dir_cova_apps_home + "/" + name + "/store/hi.lock"
		if _io_isfile pid_file, false
			_log_info "You already said hi. Say bye first."
			return true
		end

    	port = _io_findport 10080
		if port < 0
			_log_info "Too many handshakes. Try saying hi again."
			return false
		end
	
		db_file = @dir_cova_apps_home + "/" + name + "/store/log.db"		
		if not File.exist?(db_file)
 			db = SQLite3::Database.new db_file
		    db.execute("CREATE TABLE changelog(file, event, date)")
		end

		if not File.exist?(db_file)
			_log_info "Cannot start logging. Try saying hi again."
			return false
		end

		monitor = FSSM::Monitor.new
		monitor.path @dir_cova_apps_home + "/" + name do
			create do |b, r, t| 
				if File.exist?(db_file) and not r == "store/log.db" and not r == "store/hi.lock"
		 			db = SQLite3::Database.new db_file
					db.execute "insert into changelog values ('#{r}', 'created', '" + Time.new.strftime("%Y-%m-%d %H:%M:%S") + "')"
				end
			end	
			update do |b, r, t|
				if File.exist?(db_file) and not r == "store/log.db" and not r == "store/hi.lock"
		 			db = SQLite3::Database.new db_file
					db.execute "insert into changelog values ('#{r}', 'updated', '" + Time.new.strftime("%Y-%m-%d %H:%M:%S") + "')"
				end
			end	
			delete do |b, r, t|
				if File.exist?(db_file) and not r == "store/log.db" and not r == "store/hi.lock"
		 			db = SQLite3::Database.new db_file
					db.execute "insert into changelog values ('#{r}', 'deleted', '" + Time.new.strftime("%Y-%m-%d %H:%M:%S") + "')"
				end
			end
		end	

		pid = fork do
			monitor.run
		end
		# Process.detach(pid)
		File.open(pid_file, 'a') {|f| f.puts pid}
			
		Dir.chdir(@dir_cova_apps_home + "/" + name) 
		# Open3.popen3("php -S localhost:#{port} -t www &> store/server.log") {|stdin, stdout, stderr, wait_thr|
		#   puts "@@@"
		# }

	    # pid_server = fork { 
		   # `php -S localhost:10080 &> store/server.log`
		   # IO.popen("php -S localhost:10080 -t www") do |stdout|
		   # 	    puts "stdout = " + stdout.pid
		   # end
		   # Open3.popen3("php -S localhost:10080 -t www") do |stdin, stdout, stderr, wait_thr|
		   #  	puts wait_thr
		   #  	puts wait_thr.pid
		   # #  while line = stdout.gets
			  # #   puts line
			  # # end
		   #  end
		    pid_server = fork {
		    	[$stdout, $stderr].each {|fh| fh.reopen File.open("store/server.log", "a") }
		    	exec "php -S localhost:10080 -t www"
		    }
			File.open(pid_file, 'a') {|f| f.puts pid_server}
		    # cmd = 'ping google.com'
			# Open3.popen3(cmd) do |stdin, stdout, stderr, wait_thr|
			#   while line = stdout.gets
			# 	puts "@@@@@"
			#     puts line
			#   end
			# end

		   # pipe = IO.popen()
		   # File.open(pid_file, 'a') {|f| f.puts pipe.pid}
		# }
		# Process.detach(pid_server)
		# File.open(pid_file, 'a') {|f| f.puts pid_server}
		# puts status

		 # system "php", "-S", "localhost:10080", "-t", "www", "&>", "store/server.log"
		 # fork {
		  # IO.popen "php -S localhost:#{port} -t www &> store/server.log"  do |pipe|
  	 		# puts "parent pid: #{$$}, popen return (child) pid: #{pipe.pid}"
		  # end
		 # }
		 # File.open(pid_file, 'a') {|f| f.puts $?.pid}
		 # `php -S localhost:#{port} -t www &> store/server.log; echo $$ > som.pid`
		 # puts $?.pid
		# puts pid_server
		# pid_server = fork {
		# 	IO.popen "php -S localhost:10080 -t www &> t.log" do |pipe|
		# 		File.open(pid_file, 'a') {|f| f.puts pipe.pid}
		# 	end
		# }
		# Process.detach(pid_server)		

		_log_info "Hi! Checkout localhost:#{port}"
		return true
    end

    desc 'deploy NAME', 'Builds an app'
    def deploy (name)
    	if not _cova_installed
			_cova_prompt_install
			return false
		end

    	if not _io_app_exists name
    		_log_line "App #{name} does not exist"
    		_log_line_error
    		return false
    	end

    	Dir.chdir(@dir_cova_apps_home + "/" + name)
    	if not _io_isfile "app.config"
    		_log_line "App configuration file app.config does not exist."
    		_log_line_error
		    _log_info "Please add the app.config file."
		    return false
    	end

    	config = _config_load "app.config"
    	deploy_dir = config['local.www']
    	if not deploy_dir + "/#{name}"
			_log_line "Expecting the local.www path to be configured."
    		_log_line_error			
    		_log_info "Please set the local.www path in your app.config file."
		    return false
    	end

	    `rm -rf #{deploy_dir}/#{name}`
	    if _io_cpdir "www", "#{deploy_dir}/#{name}"
			_log_line "Deploying the app to #{deploy_dir}/#{name}"
			_log_line_ok
    		return true
		end
	
		_log_line "Deploying the app to #{deploy_dir}/#{name}"
		_log_line_error			
	     return false
    end

    desc 'build [-d][-t] NAME', 'Builds an app (-d to build dependencies, -t to build and run tests)'
	option :dependencies, :type => :boolean, :aliases => :d
	option :tests, :type => :boolean, :aliases => :t
    def build (name)
    	if not _cova_installed
			_cova_prompt_install
			return false
		end

    	if not _io_app_exists name
    		_log_line "App #{name} does not exist"
    		_log_line_error
    		return false
    	end

    	Dir.chdir(@dir_cova_apps_home + "/" + name + "/ios")
		_log_block_start "Building app #{name}"
     	if options[:dependencies] or not _io_isfile "Cova.xcworkspace" or not _io_isdir "Pods"
	     	if not _cova_build_dependencies
	     		_log_block_error
	     		return false
	     	end
		end

		_io_exec_xctool (options[:tests] ? "test" : "build")
		_io_open_app_source name
    end

    no_commands do

    	def _cova_prompt_install
		   _log_info "Looks like Cova is not installed yet. Please run [cova install] now"
    	end

    	def _cova_installed
    		return (File.directory?(@dir_cova_home) and File.directory?(@dir_cova_ios_mods_home))
    	end

    	def _cova_build_dependencies
			if not _io_isfile "Podfile"
				return false
			end

			_io_exec "pod repo update cova"
			_io_exec "pod install"
	     	return (_io_isfile "Cova.xcworkspace" and _io_isdir "Pods")
    	end
	end
end